@@ -18,6 +18,7 @@ struct rr;
 struct buffer;
 struct region;
 struct nsd;
+struct nsd_options;
 
 #ifdef HAVE_SYSLOG_H
 #  include <syslog.h>
@@ -303,6 +304,9 @@ extern int nsd_debug_level;
 /* set to true to log time prettyprinted, or false to print epoch */
 extern int log_time_asc;
 
+/* set to true to log time in iso format */
+extern int log_time_iso;
+
 /*
  * Timespec functions.
  */
@@ -435,4 +439,7 @@ void activate_cookie_secret(struct nsd* nsd);
 /* Drop a cookie secret. Drops the staging secret. An active secret will not
  * be dropped. */
 void drop_cookie_secret(struct nsd* nsd);
+/* Configure nsd struct with how to respond to DNS Cookies based on options */
+void reconfig_cookies(struct nsd* nsd, struct nsd_options* options);
+
 #endif /* UTIL_H */
